# ----------------------------------------------------------------------
# This file was generated by exportimages.py
#
from wx.lib.embeddedimage import PyEmbeddedImage

copy=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAHeSURBVDiN1ZXPaxNBFMc/M7OtKBUSxMZeJEgx1B9VKAqpP+hFEEQv+g941fbgzaM3PSgI"
    "RfHmxYugaMGDB6EqFdmDGMUGqqS0h6CNtqZBk2x2Z6aHNpK0u1upvfiF4Q3vvfnMvPnBCNq1"
    "BUgSrzJQjwqKlv5JILvKFyUXGA8LOCs2I6WTPTtyS6QPZHEcJywXExhm8i5jo1eOmKAxB+RX"
    "58hlo3ozR0/Re/hEJAxAOpI9/Vn2D56RKLU3coVKqa3buhJtpRZyr1Jvntw7XV380WOsla0x"
    "7fs4qlPj2DsYOxX43jXgdWvJbSoWPiSe3b16OX3wmDo0dEEiIrc18W16ctfkxNNxa+0g4IYC"
    "3z1/MJDsSYtzwzfjYAD0D52X9WpFT+dejhitXRmW9OtnaUf37oyzHqypVLpPKdXZB38OZa2E"
    "iAxFDokFblT/D9AazOYBtW7Mfy181JvBbK4wV5qdso9vD5v3Lx5SrSxsGNi82GVjzP2ZvHu8"
    "+CW3M/A871+BAPNoPebrKsC+3+XSANDxN5BaZQFjzByAisipL34vXuxKdovtyRQ68Al8b03z"
    "GzVmP71l4tGorwPvBpCLm/iSlKoG2LgmhAiQ8jorL2W9x9oBZFj+GsIUAJ+BWtOxBOMItoBE"
    "Vo2oAAAAAElFTkSuQmCC")
getcopyData=copy.GetData
getcopyImage=copy.GetImage
getcopyBitmap=copy.GetBitmap

# ----------------------------------------------------------------------
cut=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAK+SURBVDiNpdNbSBRRGAfw/8zuuLtnZlfHOd7wNkKFF0hdL0VtVhD00GtUJEVIdNEXiXqK"
    "kKAgS4IKC4Io6KnwJSgEKyqwh9TAS4RKaRJu7uw66+7s5qjrnh4yolB31f/b+c45P86Fj8P6"
    "UkgI2W+xWFINw7ixTuN3FEVpqnRXeo8dr49TqvT+P8+v0RNEkTQfPnIwZ2rKF4xEouc3BLpc"
    "rsY9e3fnxWIxDA1+GjdN891GQN7plM6UlBbbe3r6jGg0emUZi1rXcLr6nZ4d+Ywx9H7omzAM"
    "4xkACIJQK8tys82WUhOJRMeSBolELlRUlouDA0OzhhHqyMig9x0Ou6e4pDirqsot63ow/vRJ"
    "x8OkQELIgSq3W+V5Ht3d7/mq6uqmmtrqDEoVAMDkpBcvnneOB4PBu9zSnuzMzIz2WGzRY7Va"
    "GcdhyOfT6gHY0tLTTtlTbOcaTp4gXu8PhGZC2La9FppPQ29Pn398fEIzTbMzEAi0AfBxANIU"
    "qvRzAAG4xwyMZGdnHQr4A5aCgvxoYZGaY4TDXCgUhhE2QDOp/vXL2NScOfvW75++A2D4n+vI"
    "inxRLVLDAMr+1PLzcz+qqjp37fpVVl6xlbVcvsTONp1miqIMAahY7Xl4jnEWSRJ5SZLqAKiy"
    "LLcuLsZL5+fncPtW+4TD4TAdDju6ul59m56e9gDoXxXUdf2m5tNGRElskyRpOB5fbDRNs0/T"
    "/AdiCwufy8pK7aMjoyw8E34NIJTMJwIAB8ANoA6AvFQrk9PlhYLCgtjmLZtilNJ7yWLLhlL6"
    "klKqE4k0Uqq8ycvL1QC41u1JkvjT6XQ2LI33paa6ZgEUJtq4Ui+rKSk2GIYxAACKohzleT4C"
    "4HsicKVOGSMiiQiC8CjO4l4w5mGMbwUQTwRaVqjPCoJgiiLZZbVY6Pz8wgNd11sSYcmGS7zk"
    "b34Bv2kANxdpNYwAAAAASUVORK5CYII=")
getcutData=cut.GetData
getcutImage=cut.GetImage
getcutBitmap=cut.GetBitmap

# ----------------------------------------------------------------------
domain=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAJtSURBVDiNlZM/aFNBHMc/r7m2L0mb1CZp0n/YSsB2KbRLQaSDDi7dBFcVxN1BQZwEwUG6"
    "iFARhFIUHF066NDWqUqMotGhljb9E2rT/Gnz0qRJvPfOIYqU5j3sF95w9z587ne/u9OoJwD4"
    "sc8WoIBmB6YCKG2ok0vXRpkZCtLWiFIKnn/WEp88k3faI8O2wp8rCwfGWiwm2txM3Bgj4hb2"
    "S8+va30bAxO67u+xZYrZlVaDmNYkJZlCxWEjgImqmrWicoYsC1Aa0N17+uy8q7nVa8tWiu+C"
    "Wu6FW9dtmVK1upHYLMSFNzBwNTRxa0joDVsIQGpp9tz0eHJmJGQYdsz9RfTEJgjLlMvF7e8H"
    "ntBgY6NS1Mr7xuIasqOl8SlLYMtAAoiuWmr94t6TbK/Jod3qS5qqTG2OuZ4Wgrods2/kihDX"
    "RMjP5QcX1IDX4YY905vEj+p4JBA9v2vHtKy/141UXG9SGpZyPj8sC2VZliMlpQRQImnw6soc"
    "14XAbQcXpFY6LM+1pXMx8+gf158PqmUjB1TE4BluPp6it68bzU748YsZeP12N/jwnicpXJ2g"
    "hxEiAK5OXCIMIsDsy2/W7bs7PuF1E3GS/UsNKdMAiApIHQRg/qlxdMTn8vuF04NruHnqrTou"
    "NaUJpfr4hGks5e/cyYWNpZgApaPCbB6209Duha4geD3OUmS9MFEBKet3XkgT8WiarfhXEjsZ"
    "PhRLLLcIfB4Pw11Bov3dRPt7iFryuFJSILkGbxb2arGEVS3I1kMt1MFoZp8UkLEpxRUOM+nz"
    "sTcyTPmUG2FqqOwev/J5qjt5jNVV0kDtJE377/wGaRcACN5CN3MAAAAASUVORK5CYII=")
getdomainData=domain.GetData
getdomainImage=domain.GetImage
getdomainBitmap=domain.GetBitmap

# ----------------------------------------------------------------------
insert_domain=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAMrSURBVDiNfZRNaFxVGIafO/dM5tx786PNf9JghIjJJlhbELrIwi7cdCeoddG04latCApK"
    "oQUVMYhChWpBKSLWlQvFhQtTs6plTIOOXdTYJs1Mf2L+7/zfufd8LqYOScydb/UuXp7vvN/5"
    "zrGoVyfQQXxlAQGSTTwVQKzRfTxz4gAXR7to3cslAl/MWZlr7tE32/rGYoH35i8X/FvptGp1"
    "mHj5SfocFd96etHaf3t4QuuOgVhPfnU+5ZO2EmHIylalSRAgQqpRkJfmJmMAsYD+wUcen7aT"
    "KS/WW8nPdFlrXztax3qK1ertzNLWrPI6hye7J14fVXrPEQKQu/LV4fNPLVwc7/b9OM/ZX9CZ"
    "JVAmCm/k714vuN2P7k0UISht+hfm7I47fYNv6DadtG2FqYVmrHz/0uSQfz0Esj4hgOoJcotH"
    "Nj5dHYwox3W/YkllRgaOnz535HBbexuOdnG1y5kXvrMvXe05C7Dpr+Vh1lLdHTz77tMy7DXZ"
    "sAs6of7edKvVsESiZGGMQYwh5XlW78HJCkDL4q/az81qJRZGmt8fxiAioRTLBYwIIgYRQxiG"
    "kKp7wjAEELXg8+1zP3JSKZztkCAisaqVp9uTSCKpdWthpFguIA+ARgztnVG/tT71nrIV+uEg"
    "sJ8YeMs6OM4nX37Eq/v7sbYDT39mcWfcodVzcbTXmNsO7dS1oz1c7fLhye9/Vp5D324YQM2A"
    "ELI7phHBbNfGIEYQY7CTpJo8uP/mZyiVCo2Yu+F1XW8SRiGxwA4HskVo8UAQSuXijtOY+k01"
    "wGEtxF8rlGKBp44JH39TY6MAa5uQ/pPlQ6folUZkYeZcLutK4i9LtSDVRHVpbuOdHcDVdbi7"
    "DG0e9HTB2y/V9+m33+G1BSsL9FaDSiNm0q7988NU/wfK3kdmXvPi8fQ9FUaoqfNkZ/8gc3+F"
    "q/kiN1oU7a7LWE8XI0P9jAwNMMK2XQ1qAUaEKIqYv7XMT5c3gnTGVLfCVNnqfogDK5vkgJWY"
    "9HZvL0cjL/H8oVfUsZYH/40YuPZ5NK396MTNmywDAfD/dWlSbudjifftJF0AJqJSXDFnyuvk"
    "tpv+BWqctPtB+yPkAAAAAElFTkSuQmCC")
getinsert_domainData=insert_domain.GetData
getinsert_domainImage=insert_domain.GetImage
getinsert_domainBitmap=insert_domain.GetBitmap

# ----------------------------------------------------------------------
insert_layer=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAJ6SURBVDiN5ZO9axRBGMZ/szubndnL5e6SMzEBg1FREBEiIYUKET/AwkKIhY2F2PgHmFYQ"
    "G8H/wMIP0MJCFESCFoqdWqiFRvEDQTw1iURzd3t3u7c7YxHjB7mLwdanmedlZn6888wM/HcS"
    "gN7Yw6Eun1zrJQ44HrgeOIv+x4iH40nAZa4azrx5V7omd67hytl9HMz6iNZAg1AKofKgCwid"
    "Q6gCQucROg8qj9AFpkOTnDhzYZfs76Rva1872GKTISLwmapKpj43EX4DPxOyZ1sXXd0diECx"
    "brBHrspnhpwVBWMSnpUqnPO6qR4bJTk2Qv3wZk7efoSNKtiojI3KmDRBzsc0EgPyL+gXcwmD"
    "4/1ke120lmgV4Bc1RGVwPWpNS7VSrcgnsxzfe5nTjkAtB5x3GBw/Eg6H9SrGWqwxvJ2Zr4xN"
    "XL2L45Hilp+//zohZ8u8ni1zuBWko5PNQcHd6OKSJmJ3vVEbDutVrLVYa0hE+umDtRfdNKWe"
    "NJIwDGttLyPIMbJhvz+5fnu2qHyN8jV9gzmCIEOgM2gVYGMPU3fwfY1jPS6dundftgPKLme0"
    "b9gURbaOqwRSudSiEARYazHGEKgMmVUBWrsEStM/VFjdFrioJGnyMzdrFmDWLPXG0kxj2gJt"
    "KmJrFnyaptRqVawxv4EXvPnNx3FslnvQ3QPbxE1dFL0LpZCbDrhrs/kMgQrQKmBmynwrPW18"
    "kVIipUw/vQpvLXfkuY+P7Q6wAGQH3KNrx+x5qcIfHRnePGyUnl7/uuWP7P+WYSs1ojrWGuLE"
    "LJlbMdCkaWTTX98piiOSyC4hrhgYTjP58kbyQBdFEQBBOv+eOyvd/8/6DhXMJVdzbPBEAAAA"
    "AElFTkSuQmCC")
getinsert_layerData=insert_layer.GetData
getinsert_layerImage=insert_layer.GetImage
getinsert_layerBitmap=insert_layer.GetBitmap

# ----------------------------------------------------------------------
isometric=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAHSSURBVDiNxZJNSBRhGICfmZ2Zdnc2TW1TdNcVVyxi1SASEhLCQ2sHkegSHTsYSJcu4imI"
    "biF06FIWQeKhhOjgISqyqINgGBQqGqJFu+lSoCuU6zc7XydJd2fWxaKe4/vzfO/78cK/ptTc"
    "c/1gyMw2hkrXgI5dizSNeH1VYKXvbJ39Y+SETA21yQvxSLYhVLYIhN36FIdYpPaA+exInT96"
    "q7dBDZbo25IziXUuDS5Yc4nMq+S39Glgo6Bwv18VzfV7tdErTRia6rrB/ZffZf+9uexyWsSA"
    "2c14XkelqYjOijVaLo5z7eFnR9nA4yQ3H80rjeV4gNqtOccROsI2w6csvs4kiPWM83pqFYAX"
    "71c4fnmS5fkvPD+3TmtYl7m9mttKmgo9McGZKFy9/YGU0GmuUhnp+knQ79blMuFWgj4YaLfx"
    "qZI78cKyooSbOJ3DHwmL5f8JhWX9XaGu6TsXOQmldPt/p3DeGeYLVzeU7IOPurTza7dh2TC9"
    "lFEBT0FhIm0F7k5rN84/NTJvU57cNABjnzycHDbExJIxCDwp/PRvApFy/5vWakOMdiOP1Xjl"
    "bC+y67BX1OzzvQNKihXlcjRS5k1VmoodDXqTQGy3olwOFVP0C/lriBUsUM38AAAAAElFTkSu"
    "QmCC")
getisometricData=isometric.GetData
getisometricImage=isometric.GetImage
getisometricBitmap=isometric.GetBitmap

# ----------------------------------------------------------------------
orbit=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAFpSURBVDiNvdS/S1ZxFMfxV08p9gQOLoLYDx6KcHQJE3XKSQiXVofAKRvERZpc/BNaXBpL"
    "UiFycfDHkjmJjUmGJRW4+AP8Uepjwz3WJXqeLtzwAwe+33vPeZ9zvt9zL/9ZFzP6teIRPmMr"
    "L7AWz7CLHqxis5JzIQOwDTuYwzU8qeacBdiFeUzjCHV5gSW8Rxkz+JEhpqrmcDNVQF8150up"
    "dS0e4gQXsICvqPf7Esp4gW58wTr2/wYs4B7a0RKBJYzF+71UTB3u4xYasILXYb80iAmMYgjF"
    "eN6EN38UMBHdCGAvJjGQrrAG3zGLpVQbp9H+mYq4kdofSEbqAFeknGsi0wNcxxq+YRv9GJGc"
    "bSMe43mAr+ITXmEc5bMKj/AyrEFyjk1hhUhyjMs4xDKm8FGVr6aS0mNzJ5JWVJbB3kBzrJsl"
    "45ILuIbbsW7Bh7zAt+iQXODd2OcCLkpaHQ7ouwwx/1Qnnkp+tOern3aQSPwCX+wtAAAAAElF"
    "TkSuQmCC")
getorbitData=orbit.GetData
getorbitImage=orbit.GetImage
getorbitBitmap=orbit.GetBitmap

# ----------------------------------------------------------------------
pan=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AADzSURBVDiNrdQxSgRBEAXQN66Ma2LkAQQRFzyAGBt4AsXYQPAw7l5DjEwVPIC5oAjiGdRg"
    "QVjWoHtlkHGmZPvDp6Hnd1V1/emiMAZB3RG28NYnXA0GPMYM90F9J4Z4z1wrEfAC88yzPnHk"
    "yjt4QoW9pUpr4CazFyulMv4VsJb6NA6cneBcMu0HVV4PcYJTbEg9e2no9vP60NjbxQifuM68"
    "W1Q4yFwkqMQxl/7RWdvHYb7G5Nd+myljqT31P5J3BmxFcZcjw+ESB9jENm67xJEKHyU3R5L7"
    "S6OWBsMH1vvEkbf8hSupPdM+cXTATvGM16C+HL4BUo8mSicT6tEAAAAASUVORK5CYII=")
getpanData=pan.GetData
getpanImage=pan.GetImage
getpanBitmap=pan.GetBitmap

# ----------------------------------------------------------------------
paste=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAHLSURBVDiN7dS9axRBHMbx78zu3N7paXJBfDltBC3EGFLoFYKFChpOfGnEwka0sLBV8jfY"
    "CxKMhYWgIJpCJSciIhq0UMOBhV5iZUIUL4JIztvZmbEwye0WI6dtfMpnfnyYZfitIJsykJeh"
    "PK0CdR5cgDfCaKNHbGJvAnPAAoBYPFVhQC0x7AeIooih6mEGBwe83ORknfGHNdrtNlIQW8cp"
    "YCxcPD+IY9/FIUlewZWnsKeym+MnjnpBlcvx7HGNS4ckzxsuNzHlLsdJB+yLFGZzifDFFLRi"
    "uDZynbt37nnBZnOelobmD8eWElhHH0CYHlqIYewtVI6dw2iNNYkX3FgOKJbnuP36EdWdernP"
    "gLPfHNZJKkfOIuUf3mPplrMfuTFxn59aLHcyPaAtSCm7wgBClQPA2E4nPbP/nP/gigMDCdZa"
    "nLW++UxM8nuTZErJgOVegXCOxpsnXYHvX43Ts1pRUJ0us3rFCKoD8ODqMPXtuygUS17s+9cZ"
    "vnya5sxeRzu18ktgbCzCAQd2wNZ18OFzHd3y367QC/39gg1rBS+nHRI0dH6wm6SgsX4Nwaq8"
    "cF19byoz8y5ILKPGckGk+m3ASaDnLz0HvANuAckv5NKXBBlyzwYAAAAASUVORK5CYII=")
getpasteData=paste.GetData
getpasteImage=paste.GetImage
getpasteBitmap=paste.GetBitmap

# ----------------------------------------------------------------------
paste_new=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AALISURBVDiNnZTNa1RXGMZ/58y9cyeZG8zErzKmiLQR29JgwQYqFKyKSoraTXHRnS66LrT2"
    "H+im0mVBrJTSFsFCUdEqphURUReCGgIqGM1Oq+RDM8ncmbn3nPftIjGTaZlGfTbn8L4Pv/Me"
    "zsMxtKoMFGxgPwtz4eegOdrK+MxnR8TJMeAxkACY+W4Y5Bhyno8Aoihi1+BONm7sb4sbHh7h"
    "/LkhGo0G1pCKsg84Fcz3t6F8+OUuSyGE7y/B+wOb2PvJ7rbAMJ/n8oUhvtphuTKq+Wv39dvU"
    "NYE9UYhfUyK4eh9qKRw98iMnfj/ZFjg5OUUtg8lZpbcEovQABItNSQqnbsHAngP4LEO8awt8"
    "rZwjLj/mtxt/MvhOtlBvAT56qohaBj7ej7X/8x7Pp3w0xs/X/qCemYWaXWzIBKy1LwQDCMI8"
    "AF6aNdvG+8oKlrY0pSo8uHmI1/si8mGe6ngFY0yL56WA0+MPWf/BDO9t6abYGZNMdXHmhzyQ"
    "vuKEItQaVWaqFUSFeu2/niWB9dmnjN7+lSByuKTBG+Uas9UKqoK4gLVvr8D0RmwouDhJpG9J"
    "4N3hnyhtPkFcjCl2dFHsjJmpTiMqFDtiDh7fQbGzi2JHXDj8xV/fLAn0kuB8NndNUUQFEWld"
    "5/c2JGwB5iyICCqCsXOJMmYuk977hcmawEUHiJA2GtICLHcbjCqjNy+yftN2AHrX7mT08j2w"
    "DvFCT/8YWhZUhTRxnP9ujJXLAqZrgauMu9MtwDiCwX44e/hrRvrepSMuzXe6AUjrCdFqpVCa"
    "RVQgyVj19wTLJ4QL1/WZ8/zyHJh6wSiw9S1YtwLuPRkh+1cskhQMAWCo1RMkcfT2QOUZWMig"
    "GZurqSM7dFaks2C03QOpQv6uFArLrQGYuFPTkWFpPJlS65WT0PyxAd4EPgWWtQMC2BzdQcQq"
    "AJ8x5TPGgdvAccD9A477Uz6v2B3/AAAAAElFTkSuQmCC")
getpaste_newData=paste_new.GetData
getpaste_newImage=paste_new.GetImage
getpaste_newBitmap=paste_new.GetBitmap

# ----------------------------------------------------------------------
sample=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAJmSURBVDiNlZO9TxNxHIefux70hUJb2hPaIgWNCBIGZDCR0AFNTFwcHBw00UTj7p9g4mCC"
    "8SWa6Kar/4KJii8JEeliFWFAS4tguYa2tA2lvd7PoVWw9Kp8lrtf7snzfbn8JKrxAi7MkwAc"
    "Na4ZU5YGOzlzZZRngz6cjSgh4OGcZfFrx1m1vXuo08ymLUxH07HZk4rTTvjacbrtinnpp/Ot"
    "vd6BsGpz+U2ZvLYUSMdmZVnX0bLFJoMALaJSrGwXys0YUdG3ASEB/mDo6CtLi7XNDNaLm+9D"
    "yvpQh11quEMJ0PK8iyQKF5U2b99lNXxjULE1XCEAP14/CE9NxPyj3chmzM03nIgkkBSjoi/m"
    "Vr/kHWp/Y6MQlMrF7PQyVrcNXyPEEBDLkgOEcqC0EjuVfpQKVtgyq/5WFoX78fGNJ7mgMGMK"
    "+W9bMGdRVBfnb02KvrYW04lZ27Lakr3n2m3ugCmzsTSjpJfnDspCwhCmdauREEIIoykjBAIw"
    "JK+ToZEeXlgU7GZwdlvejBsjVovNY63+09+Vdt71Qmpd+/5pTOk/xPV7dwj2BnaT9TG8kryE"
    "rPiQFbX29P11Xlmj88Klu7cVhx1/c1ltJCOPof851X+lx69KTmdrV5ML9y9pA7Eosy/hjnRv"
    "hwCGUdq/sCot1DqtF9d1mNqA1SRYW8HfBR3mt3GXdJdYlFD0CsrUYxKRKJ9/pviYzxLVwel1"
    "MexxM3AwwOHJcUKnJ3DabXulyaTg5Uw+8yESj+cKpYikuhnVMiSAlEkzksPB2LEjXA0FGfa4"
    "8BgGZLJoWpr1TJb56ALPgcX/Wde+8wsuf/VmF6kkGwAAAABJRU5ErkJggg==")
getsampleData=sample.GetData
getsampleImage=sample.GetImage
getsampleBitmap=sample.GetBitmap

# ----------------------------------------------------------------------
selection=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAG3SURBVDiN3dK/axphHAbw72tPT1POH9wZkxMCiuI/4IFK/oluRx2EAxdxcBJ1yCRYhFpw"
    "LK02QYVkEFyE4NC1o1gwkM2AJINgpsRrY55OmjYVPG2H0gfe4f2+8HmfF16i/zoA9sbj8XsA"
    "+38K7dZqtW4gEPiuqqoOgNsWEhuNxlkoFHoQBAFOpxPtdvvTVlCr1TpVFEV3OBwgIhARgsHg"
    "HIBnE0hoNpvHiqLooiguISICx3Eol8tfDUOdTudDLBbTJUn6BVosr9c7H41Gh+ugF91u92M0"
    "Gv3mdrtXQkQExhgSicTtKsP0bM/6/f4OgHuLxTLneX7lxR6PB8lk8o2h5y7S6/UONE275Hn+"
    "t4aRSEQHYDPScBlZlm8mk4lV13XiuKdv5nK5KJVK9Rhj94bbATCpqnpBRJBleZ7L5T6LovhI"
    "RPD5fA+z2cxvGCMiisfjX4gIkiQ9ViqV1wBM4XD4zmw2o1AojDbC0un0ORHBbrejVCqlF/Ns"
    "NvvWarViOBy+MoxlMpkTxhhsNhuKxeLRz2f1en2vWq1eA2CGMAAvB4PBld/vRz6ff2e4xRpU"
    "nk6n2l/B/tn8AOiszSoOz6k8AAAAAElFTkSuQmCC")
getselectionData=selection.GetData
getselectionImage=selection.GetImage
getselectionBitmap=selection.GetBitmap

# ----------------------------------------------------------------------
x=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAKwSURBVDiNzZJdSFNxGMZ/+3adOadzc37VtPzApIuQlkISoReZEAZdeGEXEWFkGl1IBkIQ"
    "QVh0VyEkGBLRTRiFJWZFgiQElg5Ss7T8nE2na+6jTU8X89SyjW59rs5539/z/p/znj9sdcmk"
    "B52a5lQdtsimy0+r08uTTZ787ERaFDLkUsHhZdDtp/kvSgklBcm4hmsR7WcQ359G3GFgFjBF"
    "YKpUHSM9NWFmuBax0MxKnJKDEvD7lBD0L/po7xrHm2WAHCPcOYwlXc8DiTELXKu3kVmSAVkG"
    "eDqGb9FLuz/Ea4lRRKZcDdJrX6C6PBuTSQCrAZl9gaQpN9NrIdT5JlpuH8EgAz46oaGb8XkP"
    "x4D1f3YYobwCE2/6T2JWycHzE2xtzAXWCLw6gTVTD74Q7G/DMb5ECfAl0qyIMnBRlLHu9FJc"
    "loVGrYDCFIS9FoQD28N8XReuIQcXfSFebjZHSwggS9HR136U4tLtf/YM0P2Z0Lln9Mz8oCKq"
    "McZAAIs1gcGBU1h06nDB5YOiu8zNe8gH3NFM8mjFDQkaFXJVxFIS4iBTj1oFebFMsQYqLTo6"
    "71dh1iggsAYiIJdBRxXG5HgeAtpoxmg/BYvA9XobZZW5qAEaX7Ay6UIsSkOVoIHEOLa9m8Xq"
    "CdL534RKKLYaqKm3IQAMzCA+HmPkxlvmpja2VrMHzW4zlXFKymJ9uiQhXc/k6FnE1SbEpUbE"
    "nUnMARkqsO1Lw+lpCve+nUdMj2caSIyZ0KLj3tVDpGbow+9NvbhX/FwGpoMw8NXNo44P+AGM"
    "WrhVgSUtno6oAxPUHM81Ul6UinpiGZ6PI3aOYnd6aZUYh4eGK304hhwwsQy7klDkGilN1FIt"
    "MdI9lBk03DQJ5EiN4DrByWXqgJnIBFolJRl6LkXWvq/yaTnABcKXYYvrF1mX2jQHt3dFAAAA"
    "AElFTkSuQmCC")
getxData=x.GetData
getxImage=x.GetImage
getxBitmap=x.GetBitmap

# ----------------------------------------------------------------------
y=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAH8SURBVDiNzZI/TBNhGMZ/17v0CrRH06QFcsbEBAeJDRubi8KAsmK0NakLBhNH/8QmRicT"
    "FxxMjH82Yoz/FgeigYijg06gQ4mcg1K4lOaAO2gpbc+hd7QpXkmc+izf+7zv+zzf+735oN0h"
    "OKcc7eRJOEDULRQrlH5vcgVYbxT0BLmg+Lnk8opNVTN4DHwEkJz8rujDHB8gkYjjB/jyh8rd"
    "zxSyFskGv6DsY+rdOH2iD4plOPeSPJByG3xusGZxa3oBPRKAY2FInERUFYaBAbcn1sW926eI"
    "9EdqPS8W2SrZ3AGMA4ZAYaPI9fR8vfholJga4plDo+EAyWQcGWDZgFffWTF2eNq4kkZDrBJv"
    "ZjUyC3qNx2MwpHJCkhjpC/Lw/mmiorP1qzOsr1mkgKqnIUDWJDU5Q852+INhIt1+nvcGGRnt"
    "RwR4n6GsGcwCX5v1YnMCyAsCRyMdDA72ICkybBaRrw3RrSoIO3tw/i36iskYUGwWC/8wBOhU"
    "Q2S+TXBEkaFcBcl5S3qerelFbjbvrtWEAHs2ZHWTM2eP0+Fzrl02IP2Jpdw2Ex66gzt0YZV4"
    "/UHj51K+nrsxRy5rcRmwvXSehgBVmznNqPNVkwrwo5WmpeH/oP0NpcMa9G34tVGLd6uev2If"
    "Xv8QgJDMWG8Xky63ShRWLS4C5UNHbVv8BdrTml0zMuGBAAAAAElFTkSuQmCC")
getyData=y.GetData
getyImage=y.GetImage
getyBitmap=y.GetBitmap

# ----------------------------------------------------------------------
z=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAHGSURBVDiNzZO9S1thFMZ/NzGmzU2GGvVWYoVQkUxCJJMVF8Fa7SKoLdSAgyL+AZVmVQod"
    "XbpkKw4itpY6KIqLNNAhgh1K26EfYGibaExNvDFqbnI73II339fNZzvPe85znsN7Dlx3CPrA"
    "ZWfeVo/XaHEkxcKZwraeq9MHqsDY6hgdgkBNbH6HuR12qgoCuG8V2a6AJlt5vkDwQuHQF6S+"
    "OElRMbscNKyMIooWjfsSJ5PNkzTQu7SpJLK78YRcOoCaDqBu+cm1OAgXGypxWA6SSPBpN57e"
    "NkwA+0mYeMuvPzIPAKU431RNrNHGZHcrwzM+RICMAiMrHPyWGQLiBifUYIGuTolYYlYbUw6g"
    "DraTcN7AX62ukkNJcrD2ZpRmq1kjXoSQ96IsHp2xeCVngKXFzset8ctPePeIrGTnA2CuVVzi"
    "ULLz6lkPHffuaG/fEjC9TiQmMwDkriTYaGO63839SS83AeQLGHnNYfSEh2Bs5/RH4Wxv4PPu"
    "FM2W/22eh8gtfeLAauK8XHHslOO/mcLb1wu29rkJrz3mthEnAJ6X/IikuKvnChY7k8X889io"
    "HKhlzl4vmN5P8n54GatRwbzKV+Ptrwv+AQc3h7C1xoWhAAAAAElFTkSuQmCC")
getzData=z.GetData
getzImage=z.GetImage
getzBitmap=z.GetBitmap

# ----------------------------------------------------------------------
zoom_small=PyEmbeddedImage(
    "iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAABHNCSVQICAgIfAhkiAAAAAlw"
    "SFlzAAARTQAAEU0BwDlgYwAAABl0RVh0U29mdHdhcmUAd3d3Lmlua3NjYXBlLm9yZ5vuPBoA"
    "AAInSURBVDiNrdMxaBNRGAfw/x13RwjhCsmB2NwJpxy5dkiPJpLQLEU0EJEki5McZOrgUGiG"
    "4CQUr0UcDJRktUuHrgGjaJA4ZReaIRyIEjKcQwOp0nDWfC5NKcpda+x/e+997/e9B+8BVxzm"
    "soVEFAVwE8AJgAOGYY5m6jgYDO5alvV5dXWVFEUhXdfJNM1fzWbzHRHd+Ces1Wo9WVxcJADE"
    "siyFw+HvwWDQBUAMw1C5XP7huu7ypbBut3tH13UCQPF4/AOA66dLrCRJDyRJGgIgy7KGRBS6"
    "EKxUKjYASiaTLY+SeVEUx6FQiHq93lNfjIjmDMOY8Dw/EUUx7FWnaVoVAFWr1d50jvWoVWzb"
    "ZhRFGY1Go0Mv0LbtBgD0+/35i8BjQRDgui7vd5NIJDIHABzHuReBX5aWln4OBoNgPp/PeIGy"
    "LD8GAE3TDvwaAwC2trZeAyBd1w9TqZT453oikXjIcdxEVVVyHCfri7Xbba5QKHwFQABIluXj"
    "QqHwPB6PL6fT6Vwmk3nD8/yE53na29t764sREWOa5qcphtNHfH4MgKLRKO3u7jaJSDi//6+/"
    "vL6+/n5nZ+fedJzL5b6VSqWXnU7nkeM4ciAQcGOxWLdYLD5bWFj46Hu6jY2NVyzLnp1iZWXl"
    "qNFoXPPd5JXt7e1NQRDOMMMwxvv7+7dmwgDAcZwX2WyWAJCqqif1ev32zNg0w+GwvLa2Nq7V"
    "avf/G5uGiKQrw64ivwHZFc3aocTgCAAAAABJRU5ErkJggg==")
getzoom_smallData=zoom_small.GetData
getzoom_smallImage=zoom_small.GetImage
getzoom_smallBitmap=zoom_small.GetBitmap
